import { ReactNode } from 'react';
import { History, MemoryHistoryOptions } from 'history';
import type { Location } from '../types';
import { Route, resolve as defaultResolve } from '../resolve';
export declare type LoadStatus = {
    key: number;
    status: 'pending' | 'resolved' | 'rejected';
};
export declare type RouterContext = {
    go: History['go'];
    back: History['back'];
    forward: History['forward'];
    createHref(to: string): string;
    setView(view: ReactNode): void;
    loading?: LoadStatus;
    setLoading(loading?: LoadStatus): void;
    prefetch: (to: string, options?: {
        state?: any;
        replace?: boolean;
        done?: (view: ReactNode) => void;
        onError?: (e: Error) => void;
    }) => () => void;
    navigate: (to: string, options?: {
        state?: any;
        replace?: boolean;
    }) => void;
    refresh(): void;
    cancel(): void;
};
declare type HistoryState = {
    locationStack: Location[];
    index?: number;
} | null;
declare type Props = {
    children: ReactNode;
    routes: Route[] | Route;
    baseUrl?: string;
    resolve?: typeof defaultResolve;
    errorHandler?: (error: Error) => ReactNode;
};
export declare function BaseRouter({ routes, history, children, errorHandler, baseUrl, resolve }: Props & {
    history: History<HistoryState>;
}): JSX.Element;
declare function RouterComponent(props: Props): JSX.Element;
export { RouterComponent as Router };
export declare function HashRouter(props: Props): JSX.Element;
export declare function MemoryRouter({ initialEntries, initialIndex, ...props }: Props & MemoryHistoryOptions): JSX.Element;
export declare function useRouter(): RouterContext;
export declare function useLoading(): LoadStatus | undefined;
